/*
 * cntlr_acs.c - Auto Channel Selection header
 *
 * Copyright (C) 2021 IOPSYS Software Solutions AB. All rights reserved.
 *
 */
#ifndef CNTLR_ACS_
#define CNTLR_ACS_

#include <stdint.h>
#include <stdbool.h>

struct controller;
struct node;
struct wifi_radio_element;


struct acs_params {
	int opclass;
	int bw;

	bool skip_dfs;
	bool skip_dfs_not_available;

	int best_channel;
	int best_opclass;
	int best_bw;
	int best_pref;
};

int cntlr_acs_radio_channel_recalc(struct wifi_radio_element *radio, struct acs_params *params);
void cntlr_acs_node_channel_recalc(struct node *node, bool skip_dfs);
void cntlr_dfs_node_cleanup(struct node *node);
void cntlr_acs_recalc(struct controller *c, bool skip_dfs);
void cntlr_dfs_cleanup(struct controller *c);

int cntlr_radio_pref_opclass_add(struct wifi_radio_element *radio, uint8_t opclass,
				 uint8_t channel, uint8_t preference);
int cntlr_radio_pref_opclass_reset(struct wifi_radio_element *radio);
void cntlr_radio_pref_opclass_dump(struct wifi_radio_element *radio);

int cntlr_radio_cur_opclass_add(struct wifi_radio_element *radio, uint8_t opclass,
				uint8_t channel, uint8_t txpower);
void cntlr_radio_cur_opclass_reset(struct wifi_radio_element *radio);
void cntlr_radio_cur_opclass_dump(struct wifi_radio_element *radio);
bool cntlr_node_pref_opclass_expired(struct node *node);
void cntlr_radio_pref_opclass_set_pref(struct wifi_radio_element *radio, uint8_t id, uint8_t preference);
uint8_t ctrl_radio_cur_opclass_id(struct wifi_radio_element *radio);
uint8_t ctrl_radio_cur_opclass_ctrl_chan(struct wifi_radio_element *radio);
uint8_t ctrl_radio_cur_opclass_max_bw(struct wifi_radio_element *radio);

#endif
